package com.gic.haoban.manage.service.service.impl;

import com.alibaba.dubbo.config.annotation.Service;
import com.alibaba.fastjson.JSONObject;
import com.gic.haoban.common.utils.EntityUtil;
import com.gic.haoban.common.utils.StringUtil;
import com.gic.haoban.manage.api.dto.ExternalClerkRelatedDTO;
import com.gic.haoban.manage.api.dto.MemberUnionidRelatedDTO;
import com.gic.haoban.manage.service.dao.mapper.StaffMapper;
import com.gic.haoban.manage.service.dao.mapper.TabHaobanExternalClerkRelatedMapper;
import com.gic.haoban.manage.service.dao.mapper.TabHaobanStaffClerkRelationMapper;
import com.gic.haoban.manage.service.entity.TabHaobanExternalClerkRelated;
import com.gic.haoban.manage.service.entity.TabHaobanStaff;
import com.gic.haoban.manage.service.entity.TabHaobanStaffClerkRelation;
import com.gic.haoban.manage.service.service.ExternalClerkRelatedService;
import com.gic.member.api.dto.MemberDTO;
import com.gic.member.api.service.MemberApiService;
import com.gic.member.api.service.MemberService;
import com.gic.search.engine.api.dto.ESResponseQuerySingle;
import com.gic.search.engine.api.dto.dynamic.DynamicSearchDTO;
import com.gic.search.engine.api.service.dynamic.ESDataDynamicOperationApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;

import static org.slf4j.LoggerFactory.getLogger;

@Service
public class ExternalClerkRelatedServiceImpl implements ExternalClerkRelatedService {

    private static final Logger logger = getLogger(ExternalClerkRelatedServiceImpl.class);

    @Autowired
    private TabHaobanExternalClerkRelatedMapper mapper;

    @Autowired
    private ESDataDynamicOperationApiService esDataDynamicOperationApiService;
    @Autowired
    private MemberService memberService;
    @Autowired
    private MemberApiService memberApiService;
    @Autowired
    private StaffMapper staffMapper;
    @Autowired
    private TabHaobanStaffClerkRelationMapper staffClerkRelationMapper;

    @Override
    public void insert(TabHaobanExternalClerkRelated related) {
        String clerkId = related.getClerkId();
        String externalUserId = related.getExternalUserId();
        TabHaobanExternalClerkRelated tab = mapper.selectByClerkIdAndExternalUserId(clerkId, externalUserId);
        if (tab == null) {
            String uuId = StringUtil.randomUUID();
            related.setStatusFlag(related.getStatusFlag() != null ? related.getStatusFlag() : 1);
            related.setExternalClerkRelatedId(uuId);
            related.setCreateTime(new Date());
            related.setUpdateTime(new Date());
            mapper.insertSelective(related);
        }


    }

    @Override
    public void delByUserIdAndExternalUserId(MemberUnionidRelatedDTO dto) {
        mapper.delByUserIdAndExternalUserId(EntityUtil.changeEntityByJSON(TabHaobanExternalClerkRelated.class, dto));
        String wxUserId = dto.getWxUserId();
        List<TabHaobanStaff> tabHaobanStaffs = staffMapper.listByWxUserId(wxUserId);
        if (CollectionUtils.isEmpty(tabHaobanStaffs)) {
            logger.error("找不到员工:wxUserId:{}", wxUserId);
            return;
        }
        String wxEnterpriseId = tabHaobanStaffs.get(0).getWxEnterpriseId();
        String staffId = tabHaobanStaffs.get(0).getStaffId();
        List<String> statusFlags = new ArrayList<>();
        statusFlags.add("1");
        statusFlags.add("3");
        statusFlags.add("4");
        //该企微该会员所有关联关系解除后通知会员
        List<TabHaobanExternalClerkRelated> list = mapper.listByExTernalUseridAndWxEnterpriseId(dto.getExternalUserid(), wxEnterpriseId, statusFlags);
        if (CollectionUtils.isEmpty(list)) {
            List<TabHaobanStaffClerkRelation> tabHaobanStaffClerkRelations = staffClerkRelationMapper.lisByStaffId(wxEnterpriseId, staffId);
            if (CollectionUtils.isEmpty(tabHaobanStaffClerkRelations)) {
                logger.error("找不到员工导购关联关系:staffId:{}，wxEnterpriseId：{}", staffId, wxEnterpriseId);
                return;
            }
            String enterpriseId = tabHaobanStaffClerkRelations.get(0).getEnterpriseId();
            List<String> statusFlagList = new ArrayList<>();
            statusFlagList.add("2");
            //删除的好友关系
            List<TabHaobanExternalClerkRelated> haobanExternalClerkRelatedList = mapper.listByExTernalUseridAndWxEnterpriseId(dto.getExternalUserid(), wxEnterpriseId, statusFlagList);
            for (TabHaobanExternalClerkRelated externalClerkRelated : haobanExternalClerkRelatedList) {
                memberApiService.updateMemberQywxFlag(enterpriseId, externalClerkRelated.getMemberId(), 0);
            }
        }
    }

    @Override
    public void delByUserIdAndExternalUserIdBatch(String wxEnterpriseId, String wxUserId, List<String> externalUserids) {
        mapper.delByUserIdAndExternalUserIdBatch(wxUserId, wxEnterpriseId, externalUserids);
    }

    @Override
    public void delByClerkIdAndStaffId(String clerkId, String staffId) {
        mapper.delByClerkIdAndStaffId(clerkId, staffId);
    }

    @Override
    public void delByStoreIdAndStaffId(String storeId, String staffId) {
        mapper.delByStoreIdAndStaffId(storeId, staffId);
    }

    @Override
    public void delByStoreIds(Collection<String> storeIds) {
        mapper.delByStoreIds(storeIds);
    }

    @Override
    public void cleanStaffRelated(String wxEnterpriseId, List<String> staffIds) {
        if (staffIds != null && !staffIds.isEmpty()) {
            mapper.cleanStaffRelated(wxEnterpriseId, staffIds);
        }

    }

    @Override
    public void update(TabHaobanExternalClerkRelated exsitDTO) {
        exsitDTO.setUpdateTime(new Date());
        mapper.updateByPrimaryKeySelective(exsitDTO);

    }

    @Override
    public TabHaobanExternalClerkRelated getByParams(String userId, String wxEnterpriseId, String name,
                                                     String createTime) {
        return mapper.getByParams(userId, wxEnterpriseId, name, createTime);
    }

    @Override
    public List<TabHaobanExternalClerkRelated> listByExTernalUseridAndWxUserId(String externalUserid, String wxUserId) {
        return mapper.listByExTernalUseridAndWxUserId(externalUserid, wxUserId);
    }

    @Override
    public List<TabHaobanExternalClerkRelated> listByExTernalUseridAndWxUserIdAllStatus(String wxEnterpriseId, String externalUserid, String wxUserId) {
        return mapper.listByExTernalUseridAndWxUserIdAllStatus(wxEnterpriseId, externalUserid, wxUserId);
    }

    @Override
    public ExternalClerkRelatedDTO getByMemberId(String wxUserId, String memberId) {
        return EntityUtil.changeEntityByJSON(ExternalClerkRelatedDTO.class, mapper.getByMemberId(wxUserId, memberId));
    }

    @Override
    public List<TabHaobanExternalClerkRelated> listExternalByUnionId(String enterpriseId, String unionId) {
        return mapper.listExternalByUnionId(enterpriseId, unionId);
    }

    @Override
    public List<TabHaobanExternalClerkRelated> listExternalByExternalUserid(String wxEnterpriseId, String externalUserid) {
        return mapper.listByExternalUserId(wxEnterpriseId, externalUserid);
    }

    @Override
    public String checkAndUpdateMemberByUnonId(String enterpriseId, String memberId, String unionId) {
        String returnFields = "id";
        try {
            JSONObject searchJson = new JSONObject();
            searchJson.put("id", memberId);
            DynamicSearchDTO search = new DynamicSearchDTO();
            search.setSearchJson(searchJson);
            search.setEnterpriseId(enterpriseId);
            search.setColumnCategoryCode("member");
            if (StringUtils.isNotBlank(returnFields)) {
                search.setReturnFileds(returnFields);
            }
            ESResponseQuerySingle single = esDataDynamicOperationApiService.queryDataSingle(search);
            JSONObject res = single.getRes();
            logger.info("返回数据：{}", JSONObject.toJSONString(res));
            if (res != null && res.size() > 0) {
                return memberId;
            }
            MemberDTO memberDTO = memberService.getMemberByUnionid(unionId, enterpriseId);
            if (null == memberDTO || memberDTO.getMemberId() == null) {
                logger.info("会员不存在：{}", unionId);
                return null;
            }
            //更新会员id
            mapper.updateMemberIdByMemberId(memberDTO.getMemberId(), memberId);
            return memberDTO.getMemberId();
        } catch (Exception e) {
            logger.info("从搜索引擎中搜索会员数据异常, memberId : {}", memberId);
        }
        return null;
    }
}
