package com.gic.haoban.manage.service.service.notify.out;

import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.BasePageInfo;
import com.gic.api.base.commons.Constant;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.clerk.api.service.ClerkService;
import com.gic.commons.util.PageHelperUtils;
import com.gic.haoban.manage.api.dto.StaffClerkRelationDTO;
import com.gic.haoban.manage.api.dto.WxEnterpriseDTO;
import com.gic.haoban.manage.api.dto.notify.dto.NoticeMessageInfoDTO;
import com.gic.haoban.manage.api.dto.notify.qdto.NoticeMessageQDTO;
import com.gic.haoban.manage.api.enums.NoticeMessageTypeEnum;
import com.gic.haoban.manage.api.service.notify.NoticeMessageApiService;
import com.gic.haoban.manage.service.config.Config;
import com.gic.haoban.manage.service.entity.TabHaobanStaff;
import com.gic.haoban.manage.service.errorcode.HaoBanErrCode;
import com.gic.haoban.manage.service.pojo.bo.NoticeMessageBO;
import com.gic.haoban.manage.service.pojo.bo.TemplateContentBO;
import com.gic.haoban.manage.service.service.StaffClerkRelationService;
import com.gic.haoban.manage.service.service.StaffService;
import com.gic.haoban.manage.service.service.WxEnterpriseService;
import com.gic.haoban.manage.service.service.notify.NoticeMessageService;
import com.gic.haoban.manage.service.service.notify.NoticeMessageTemplateService;
import com.gic.wechat.api.dto.qywx.ItemDTO;
import com.gic.wechat.api.dto.qywx.QywxXcxSendMessageDTO;
import com.gic.wechat.api.service.qywx.QywxSuiteApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static org.slf4j.LoggerFactory.getLogger;

/**
 * Created 2021/12/17.
 *
 * @author hua
 */
@Service
public class NoticeMessageApiServiceImpl implements NoticeMessageApiService {

    private static final Logger logger = getLogger(NoticeMessageApiServiceImpl.class);

    @Autowired
    private ClerkService clerkService;

    @Autowired
    private NoticeMessageTemplateService noticeMessageTemplateService;

    @Autowired
    private Config config;

    @Autowired
    private StaffClerkRelationService staffClerkRelationService;

    @Autowired
    private NoticeMessageService noticeMessageService;

    @Autowired
    private QywxSuiteApiService qywxSuiteApiService;

    @Autowired
    private WxEnterpriseService wxEnterpriseService;

    @Autowired
    private StaffService staffService;

    @Override
    public void noticeMessageMq(String json) {
        logger.info("通知消息：{}", json);
        NoticeMessageQDTO messageQDTO = JSONObject.parseObject(json, NoticeMessageQDTO.class);
        NoticeMessageTypeEnum messageTypeEnum = NoticeMessageTypeEnum.getByType(messageQDTO.getMessageType());
        messageQDTO.setTemplateCode(messageTypeEnum.getTemplateCode());
        //企业微信模板
        List<TemplateContentBO> contentList = getDescriotion(messageQDTO);
        //好办模板
        String haobanNotice = getHaobanNotice(messageQDTO);
        //模板不存在
        if (CollectionUtils.isEmpty(contentList) || StringUtils.isBlank(haobanNotice)) {
            logger.info("没有模板，无法发送消息：{}", messageQDTO.getTemplateCode());
            return;
        }
        //stff存在 发送到staff上
        String staffId = messageQDTO.getStaffId();
        String wxUserId = null;
        String wxEnterpriseId = null;
        String storeId = "-1";
//        if (StringUtils.isNotEmpty(messageQDTO.getClerkId())) {
//            StaffClerkRelationDTO clerkRelationDTO = staffClerkRelationService.getByClerkId(messageQDTO.getClerkId());
//            staffId = staffId == null ? clerkRelationDTO.getStaffId() : staffId;
//            storeId = clerkRelationDTO.getStoreId();
//        }


        StaffClerkRelationDTO clerkRelationDTO = null;
        if (StringUtils.isNotBlank(messageQDTO.getStaffId())) {
            TabHaobanStaff haobanStaff = staffService.selectById(messageQDTO.getStaffId());
            wxUserId = haobanStaff != null ? haobanStaff.getWxUserId() : null;
            wxEnterpriseId = haobanStaff != null ? haobanStaff.getWxEnterpriseId() : null;
        }
        if (StringUtils.isNotEmpty(messageQDTO.getClerkId())) {
            clerkRelationDTO = staffClerkRelationService.getByClerkId(messageQDTO.getClerkId());
        }
        if (wxUserId == null && clerkRelationDTO == null) {
            logger.info("没有导购：{}", messageQDTO.getClerkId());
            return;
        }
        wxUserId = (wxUserId == null ? clerkRelationDTO.getWxUserId() : wxUserId);
        wxEnterpriseId = (wxEnterpriseId == null ? clerkRelationDTO.getWxEnterpriseId() : wxEnterpriseId);

        //消息组装 新增消息
        NoticeMessageBO noticeMessageBO = new NoticeMessageBO();
        noticeMessageBO.setCategoryType(messageTypeEnum.getCategory());
        noticeMessageBO.setMessageType(messageQDTO.getMessageType());
        noticeMessageBO.setClerkId(messageQDTO.getClerkId() != null ? messageQDTO.getClerkId() : "-1");
        noticeMessageBO.setStoreId(clerkRelationDTO != null ? clerkRelationDTO.getStoreId() : "-1");
        noticeMessageBO.setTitle(messageTypeEnum.getName());
        noticeMessageBO.setTemplateCode(messageTypeEnum.getTemplateCode());
        noticeMessageBO.setDescription(haobanNotice);
        noticeMessageBO.setEnterpriseId(messageQDTO.getEnterpriseId());
        noticeMessageBO.setRelationId(messageQDTO.getOptTargetId());
        noticeMessageBO.setMessageContent(messageQDTO.getExtendContent() != null ? JSONObject.toJSONString(messageQDTO.getExtendContent()) : null);
        noticeMessageService.addOrCreateNoticeMessage(noticeMessageBO);

        //发送企业微信
        sendMessage(contentList, wxEnterpriseId, wxUserId, messageTypeEnum, noticeMessageBO.getMessageContent());
    }

    @Override
    public void run(String json) {
        this.noticeMessageMq(json);
    }

    @Override
    public ServiceResponse<Page<NoticeMessageInfoDTO>> pageNoticeMessage(String enterpriseId, String storeId, String clerkId, int categoryType, BasePageInfo pageInfo) {
        if (StringUtils.isAnyBlank(enterpriseId, storeId)) {
            ServiceResponse.failure(HaoBanErrCode.ERR_0001.getCode(), HaoBanErrCode.ERR_0001.getMsg());
        }
        Page<NoticeMessageBO> retPage = noticeMessageService.pageNoticeMessage(enterpriseId, storeId, clerkId, categoryType, pageInfo);
        Page<NoticeMessageInfoDTO> tPage = PageHelperUtils.changePageToCurrentPage(retPage, NoticeMessageInfoDTO.class);
        if (retPage != null && CollectionUtils.isNotEmpty(tPage.getResult())) {
            tPage.getResult().forEach(mid -> {
                NoticeMessageTypeEnum messageTypeEnum = NoticeMessageTypeEnum.getByType(mid.getMessageType());
                mid.setPageType(messageTypeEnum.getPageType());
            });
        }
        return ServiceResponse.success(tPage);
    }

    @Override
    public ServiceResponse<Integer> countUnReadNoticeMessage(String enterpriseId, String storeId, String clerkId) {
        int count = noticeMessageService.countUnreadNoticeMessage(enterpriseId, storeId, clerkId);
        return ServiceResponse.success(count);
    }

    @Override
    public ServiceResponse<Boolean> updateReadFlagById(Long id) {
        NoticeMessageBO noticeMessageBO = new NoticeMessageBO();
        noticeMessageBO.setNoticeMessageId(id);
        noticeMessageBO.setReadFlag(Constant.NUMBER_1);
        noticeMessageService.addOrCreateNoticeMessage(noticeMessageBO);
        return ServiceResponse.success(true);
    }

    @Override
    public ServiceResponse<Boolean> updateWriteFlagByClerkId(String enterpriseId, String clerkId, int categoryType) {
        noticeMessageService.updateNoticeMessageReadFlag(enterpriseId, clerkId, categoryType);
        return ServiceResponse.success(true);
    }

    /**
     * 获取详情
     *
     * @param messageQDTO
     * @return
     */
    private List<TemplateContentBO> getDescriotion(NoticeMessageQDTO messageQDTO) {
        List<TemplateContentBO> templateMessage = noticeMessageTemplateService.getQywxTemplateMessage(messageQDTO.getEnterpriseId(), messageQDTO.getTemplateCode());
        if (CollectionUtils.isEmpty(templateMessage)) {
            //todo 需要告警
            logger.info("不存在模板");
            return null;
        }


        //需要替换的数据
        Map<String, String> contentMap = messageQDTO.getContentMap();
        if (MapUtils.isEmpty(contentMap)) {
            return templateMessage;
        }

        //替换的内容
        for (TemplateContentBO templateContentBO : templateMessage) {

            contentMap.forEach((k, v) -> {
                templateContentBO.setVal(templateContentBO.getVal().replaceAll("\\#\\{" + k + "\\}", v));
            });
        }
        return templateMessage;
    }


    /**
     * 获取详情好办的通知
     *
     * @param messageQDTO
     * @return
     */
    private String getHaobanNotice(NoticeMessageQDTO messageQDTO) {
        final String[] haobanTemplateMessage = {noticeMessageTemplateService.getHaobanTemplateMessage(messageQDTO.getEnterpriseId(), messageQDTO.getTemplateCode())};
        if (StringUtils.isBlank(haobanTemplateMessage[0])) {
            //todo 需要告警
            logger.info("不存在好办的通知模板模板");
            return null;
        }
        //需要替换的数据
        Map<String, String> contentMap = messageQDTO.getContentMap();
        if (MapUtils.isEmpty(contentMap)) {
            return haobanTemplateMessage[0];
        }
        //替换的内容
        contentMap.forEach((k, v) -> {
            haobanTemplateMessage[0] = haobanTemplateMessage[0].replaceAll("\\#\\{" + k + "\\}", v);
        });
        return haobanTemplateMessage[0];
    }

    /**
     * 发送消息
     *
     * @param contentList
     * @param wxUserId
     * @param messageTypeEnum
     */
    private void sendMessage(List<TemplateContentBO> contentList, String wxEnterpriseId, String wxUserId, NoticeMessageTypeEnum messageTypeEnum, String data) {
        WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseService.selectById(wxEnterpriseId);
        QywxXcxSendMessageDTO messageDTO = new QywxXcxSendMessageDTO();

        String corpid = wxEnterpriseDTO.getCorpid();

        TabHaobanStaff tabHaobanStaff = staffService.selectByUserIdAndEnterpriseId(wxUserId, wxEnterpriseId);

        if (corpid.length() > 20) {
            wxUserId = tabHaobanStaff.getWxOpenUseId();
        } else {
            wxUserId = tabHaobanStaff.getWxUserId();
        }

        //后续map加判断可加参数
        Map<String, String> map = new HashMap<>();
        List<ItemDTO> items = contentList.stream().map(bo -> {
            ItemDTO item = new ItemDTO();
            item.setKey(bo.getKey());
            item.setValue(bo.getVal());
            return item;
        }).collect(Collectors.toList());
        if (StringUtils.isNotBlank(data)) {
            try {
                data = URLEncoder.encode(data, StandardCharsets.UTF_8.name());
            } catch (UnsupportedEncodingException e) {
                logger.error(e.getMessage());
            }
        }
        String appUrl = messageTypeEnum.getPageUrl();
        if (StringUtils.isNotBlank(messageTypeEnum.getPageType())) {
            appUrl += messageTypeEnum.getPageType();
        }
        appUrl += "&data=" + data;

        List<String> userList = new ArrayList<>();
        userList.add(wxUserId);
        messageDTO.setAppid(config.getAppid());
        messageDTO.setUserIds(userList);
        messageDTO.setPage(appUrl);
        messageDTO.setTitle(messageTypeEnum.getName());
        messageDTO.setItems(items);
        qywxSuiteApiService.sendMessage(wxEnterpriseDTO.getCorpid(), config.getWxSuiteid(), messageDTO);
    }
}
