package com.gic.haoban.manage.service.task;

import com.alibaba.fastjson.JSONObject;
import com.gic.clerk.api.dto.ClerkListDTO;
import com.gic.clerk.api.service.ClerkService;
import com.gic.commons.util.GICMQClientUtil;
import com.gic.enterprise.api.dto.StoreDTO;
import com.gic.enterprise.api.service.StoreService;
import com.gic.haoban.base.api.common.Constant;
import com.gic.haoban.base.api.common.ServiceResponse;
import com.gic.haoban.manage.api.dto.DealParamMqDTO;
import com.gic.haoban.manage.api.dto.DepartmentDTO;
import com.gic.haoban.manage.api.enums.PreDealStatusEnum;
import com.gic.haoban.manage.api.enums.PreDealTypeEnum;
import com.gic.haoban.manage.api.enums.SyncTaskStatusEnum;
import com.gic.haoban.manage.api.service.DepartmentApiService;
import com.gic.haoban.manage.service.entity.TabHaobanDepartment;
import com.gic.haoban.manage.service.entity.TabHaobanPreDealLog;
import com.gic.haoban.manage.service.entity.TabHaobanSyncTask;
import com.gic.haoban.manage.service.service.DepartmentService;
import com.gic.haoban.manage.service.service.PreDealService;
import com.gic.haoban.manage.service.service.SyncTaskService;
import com.gic.mq.sdk.GicMQClient;
import com.gic.redis.data.util.RedisUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * Created 2020/4/13.
 *
 * @author hua
 */
@Service
@Deprecated
public class StoreSyncOperation implements BaseSyncOperation {
    private static final Logger logger = LoggerFactory.getLogger(StoreSyncOperation.class);

    @Autowired
    private DepartmentService departmentService;

    @Autowired
    private PreDealService preDealService;

    @Autowired
    private DepartmentApiService departmentApiService;

    @Autowired
    private SyncTaskService syncTaskService;

    @Autowired
    private StoreService storeService;

    @Autowired
    private ClerkService clerkService;

    @Autowired
    private ClerkSyncOperation clerkSyncOperation;

    @Override
    public void dealSingleByMq(DealParamMqDTO dealParamMqDTO, TabHaobanPreDealLog dataPre) {
        logger.info("处理单条门店：{}", JSONObject.toJSONString(dealParamMqDTO));
        DepartmentDTO departmentDTO = departmentApiService.selectByRelatedId(dataPre.getDataId());
        String reason = "";
        if (departmentDTO != null) {
            logger.info("已经同步过了:{}", dataPre.getDataId());
            dealException(dealParamMqDTO.getTaskId(), dealParamMqDTO.getData(), dataPre.getEnterpriseId(), "该部门已经同步");
            return;
        }

        TabHaobanDepartment pParentDepartment = departmentService.selectByRelatedId(dataPre.getpDataId());
        if (null == pParentDepartment) {
            pParentDepartment = departmentService.selectById(dataPre.getpDataId());
        }
        if (null == pParentDepartment) {
            logger.info("不存在该父部门：c:{},p:{}", dataPre.getDataId(), dataPre.getpDataId());
            dealException(dealParamMqDTO.getTaskId(), dealParamMqDTO.getData(), dataPre.getEnterpriseId(), "父部们不存在");
            return;
        }
        StoreDTO storeDTO = storeService.getStore(dealParamMqDTO.getData());
        if (null == storeDTO) {
            logger.info("该部门不存在");
            dealException(dealParamMqDTO.getTaskId(), dealParamMqDTO.getData(), dataPre.getEnterpriseId(), "该部门不存在");
            return;
        }
        DepartmentDTO department = new DepartmentDTO();
        department.setParentDepartmentId(pParentDepartment.getDepartmentId());
        department.setWxEnterpriseId(pParentDepartment.getWxEnterpriseId());
        department.setDepartmentName(storeDTO.getStoreName());
        department.setChainId(pParentDepartment.getChainId() + Constant.ID_SEPARATOR + pParentDepartment.getDepartmentId());
        department.setChainName(pParentDepartment.getChainName() + Constant.NAME_SEPARATOR + pParentDepartment.getDepartmentName());
        department.setIsStore(1);
        department.setStoreCode(storeDTO.getStoreCode());
        department.setRelatedCode(storeDTO.getStoreCode());
        department.setLevel(pParentDepartment.getLevel() + 1);
        department.setRelatedId(dealParamMqDTO.getData());
        //状态 处理状态
        boolean resultFlag = true;

        try {
            ServiceResponse add = departmentApiService.add(department);
            if (add.getCode() != 1) {
                resultFlag = false;
                reason = add.getMessage();
                logger.info("门店处理处理失败：{}", dealParamMqDTO.getData());
                return;
            }
        } catch (Exception e) {
            resultFlag = false;
            e.printStackTrace();
            logger.info("处理异常");
        } finally {
            if (!resultFlag) {
                dealException(dealParamMqDTO.getTaskId(), dealParamMqDTO.getData(), dataPre.getEnterpriseId(), reason);
            } else {
                dealSuccess(dealParamMqDTO.getTaskId(), dealParamMqDTO.getData(), dataPre.getEnterpriseId(), dataPre.getWxEnterpriseId());
            }
        }
    }

    @Override
    public void dealException(String taskId, String dataId, String enterpriseId, String reason) {
        //处理异常
        preDealService.updateStatusByDataId(taskId, dataId, PreDealStatusEnum.exception.getVal(), reason);
        checkDepartmentTask(taskId);
    }

    @Override
    public void dealSuccess(String taskId, String dataId, String enterpriseId, String wxEnterpriseId) {

        List<ClerkListDTO> clerks = clerkService.getClerkByStoreId(enterpriseId, dataId);
        List<TabHaobanPreDealLog> hasExists = preDealService.listByPDataId(taskId, dataId, -1);
        Set<String> hasDatas = new HashSet<>();
        if (CollectionUtils.isNotEmpty(hasExists)) {
            hasDatas = hasExists.stream().map(mid -> mid.getDataId()).collect(Collectors.toSet());
        }
        logger.info("导入clerk:{},{},{}", enterpriseId, dataId, JSONObject.toJSONString(clerks));
        //预处理门店 写入预处理表
        if (CollectionUtils.isNotEmpty(clerks)) {
            Set<String> finalHasDatas = hasDatas;
            List<TabHaobanPreDealLog> preDealLogList = clerks.stream().filter(mid -> {
                if (finalHasDatas.contains(mid.getClerkId())) {
                    return false;
                }
                return !StringUtils.isBlank(mid.getPhoneNumber());
            }).map(clerkMid -> {
                TabHaobanPreDealLog dealLog = new TabHaobanPreDealLog();
                dealLog.setDataId(clerkMid.getClerkId());
                dealLog.setpDataId(dataId);
                dealLog.setDataType(PreDealTypeEnum.clerk.getVal());
                dealLog.setStatusFlag(0);
                dealLog.setTaskId(taskId);
                dealLog.setEnterpriseId(clerkMid.getEnterpriseId());
                dealLog.setWxEnterpriseId(wxEnterpriseId);
                return dealLog;
            }).collect(Collectors.toList());
            logger.info("过滤后导入clerk:{},{},{}", enterpriseId, dataId, JSONObject.toJSONString(preDealLogList));
            preDealService.insert(preDealLogList);
        }
        boolean b = preDealService.updateStatusByDataId(taskId, dataId, PreDealStatusEnum.computed.getVal(), "成功");
        if (!b) {
            logger.info("成功修改错误！{}", dataId);
            return;
        }
        checkDepartmentTask(taskId);

    }

    @Override
    public void checkDepartmentTask(String taskId) {
        boolean b = preDealService.checkTask(taskId, PreDealTypeEnum.store.getVal());
        if (!b) {
            return;
        }
        logger.info("门店继续成功，成员同步:{}", taskId);
        String key = "haoban_sync_store_task_" + taskId;
        RedisUtil.lock(key, 3L);
        TabHaobanSyncTask syncTask = syncTaskService.getSyncTask(taskId);
        //同步成功 进入门店处理
        if (syncTask.getStatusFlag().equals(SyncTaskStatusEnum.store_sync.getVal())) {
            syncTaskService.updateTaskStatus(taskId, SyncTaskStatusEnum.clerk_sync.getVal());
            //是否有成员 需要处理
            List<TabHaobanPreDealLog> dealLogs = preDealService.listByTaskId(taskId, PreDealTypeEnum.clerk.getVal(), PreDealStatusEnum.pre.getVal());
            if (CollectionUtils.isEmpty(dealLogs)) {
                clerkSyncOperation.checkDepartmentTask(taskId);
//                syncTaskService.updateTaskStatus(taskId, SyncTaskStatusEnum.compute.getVal());
//                dealSyncOperationApiService.unlockTask(syncTask.getWxEnterpriseId());
            } else {
                logger.info("处理成员开始：{}", taskId);
                Set<String> dataIds = dealLogs.stream().map(TabHaobanPreDealLog::getDataId).collect(Collectors.toSet());
                dealDepartmentToMq(taskId, dataIds, SyncTaskStatusEnum.clerk_sync);
            }
        }
        RedisUtil.unlock(key);
    }


    /**
     * 放入mq处理部门数据
     *
     * @param taskId
     * @param dealList
     */
    private void dealDepartmentToMq(String taskId, Set<String> dealList, SyncTaskStatusEnum syncTaskStatusEnum) {
        //预处理分组任务
        syncTaskService.updateTaskStatus(taskId, syncTaskStatusEnum.getVal());
        dealList.forEach(relationId -> {
            DealParamMqDTO dealParamMqDTO = new DealParamMqDTO();
            dealParamMqDTO.setTaskId(taskId);
            dealParamMqDTO.setData(relationId);
            dealParamMqDTO.setType(syncTaskStatusEnum.getVal());
            GicMQClient clientInstance = GICMQClientUtil.getClientInstance();
            try {
                clientInstance.sendMessage("departmentSyncDealMq", JSONObject.toJSONString(dealParamMqDTO));
            } catch (Exception e) {
                logger.info("发送失败:{},{}", taskId, relationId);
                e.printStackTrace();
            }
        });
    }
}
